"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _trieCodec = _interopRequireDefault(require("@polkadot/trie-codec"));

var _constants = require("@polkadot/trie-codec/constants");

var _util = require("@polkadot/util");

var _Cache = _interopRequireDefault(require("./Cache"));

var _Impl = _interopRequireDefault(require("./Impl"));

var _defaults = _interopRequireDefault(require("./defaults"));

var _util2 = require("./util");

// Copyright 2017-2019 @polkadot/client-db authors & contributors
// This software may be modified and distributed under the terms
// of the Apache-2.0 license. See the LICENSE file for details.
const l = (0, _util.logger)('db/struct');

class FileStructDb extends _Impl.default {
  constructor() {
    super(...arguments);
    this._cache = new _Cache.default(16 * 1024);
  }

  drop() {
    l.error('drop() is not implemented');
  }

  empty() {
    l.error('empty() is not implemented');
  }

  rename(base, file) {
    l.error('rename() is not implemented');
  }

  size() {
    l.error('size() is not implemented');
    return 0;
  }

  maintain(fn) {
    fn({
      isCompleted: true,
      keys: 0,
      percent: 100
    });
  }

  del(key) {
    l.error('del() is not implemented');
  }

  _get(key) {
    const keyInfo = this._findValue((0, _util2.serializeKey)(key));

    if (!keyInfo || !keyInfo.valData) {
      return null;
    } else if (!this._isTrie) {
      return keyInfo.valData;
    }

    const bitmap = (keyInfo.valData[0] << 8) + keyInfo.valData[1];
    let offset = 2;

    if (!bitmap) {
      return keyInfo.valData.subarray(offset);
    }

    const recoded = [];

    for (let index = 0; index < 16; index++) {
      if (bitmap & _constants.BITMAP[index]) {
        const keyBuff = this._readKey(0, (0, _util2.readU8aU32)(keyInfo.valData, offset));

        recoded.push(keyBuff.subarray(0, _defaults.default.KEY_DATA_SIZE));
        offset += _defaults.default.U32_SIZE;
      } else {
        recoded.push(null);
      }
    }

    recoded.push(null);
    return _trieCodec.default.encode(recoded);
  }

  get(key) {
    // l.debug(() => ['get', { key }]);
    const keyStr = key.toString();

    const cached = this._cache.get(keyStr);

    if (cached) {
      return cached;
    }

    const value = this._get(key);

    if (value) {
      this._cache.set(keyStr, value);
    }

    return value;
  }

  put(key, value) {
    // l.debug(() => ['put', { key, value }]);
    let adjusted = null;

    if (this._isTrie) {
      const decoded = _trieCodec.default.decode(value); // extension nodes are going away anyway, so just ignore, no worse off.


      if (Array.isArray(decoded) && decoded.length === 17) {
        // only deal with keys where it is an actual trie hash
        const hasArrays = decoded.some(value => Array.isArray(value) || value && value.length !== 32); // great we are dealing with keys only here

        if (!hasArrays && !decoded[16]) {
          let bitmap = 0;
          const recoded = [0, 0];

          for (let index = 0; index < 16; index++) {
            const u8a = decoded[index];

            if (u8a) {
              const entry = (0, _util2.serializeKey)(u8a);

              const keyInfo = this._findValue(entry, null, false);

              bitmap |= _constants.BITMAP[index];
              recoded.push(...(0, _util2.u32ToArray)(keyInfo.keyAt));
            }
          }

          recoded[0] = bitmap >> 8;
          recoded[1] = bitmap & 255;
          adjusted = new Uint8Array(recoded);
        }
      }

      if (!adjusted) {
        adjusted = new Uint8Array(value.length + 2);
        adjusted.set(value, 2);
      }
    }

    this._cache.set(key.toString(), value);

    this._findValue((0, _util2.serializeKey)(key), adjusted || value);
  }

}

exports.default = FileStructDb;